function cstr = M11C11T01P21(us,perf,varsglob,varsloc)
% M11C11T01P21 - LFT model - State feedback- Quadratic stability - Hinfinity test  
  
%   This file is part of RoMulOC
%   Last Update 7-Nov-2014
%   author : Dimitri Peaucelle
%   peaucelle@laas.fr
%   LAAS-CNRS, Toulouse, France

n=us.n;
mu=us.mu;
mw=us.mw;
md=us.md;
pz=us.pz;
pd=us.pd;

P = varsglob{2,1};
S = varsglob{3,1};
sepvar = varsloc{2,1};

if isa(P,'sdpvar')
  isLMI=1;
  cstr=lmi;
else
  isLMI=0;
  cstr=cell(1,1);
end

if perf
  gamma2 = perf^2;
  tau = varsloc{1,1};
else
  gamma2 = varsloc{1,1};
  tau = 1;
end

RS = [zeros(mu,mu), S;...
      S', zeros(n, n)];
deltanul = ~any(any([us.Bd; us.Dzd]));

if us.T == 0
  RP = [zeros(n,n), P;...
        P, zeros(n, n)];
  if deltanul
    MP = [us.A, eye(n,n);...
          us.Cz, zeros(pz,n)];
    MS = [us.Bu, eye(n,n);...
          us.Dzu, zeros(pz,n)];
    Mg = [zeros(n, pz);...
          eye(pz, pz)];
    Mc = [us.Bw;...
          us.Dzw];
    lyap = MP*RP*MP' + MS*RS*MS' - tau*gamma2*(Mg*Mg') + tau*(Mc*Mc');
  else
    MP = [us.A, eye(n,n);...
          us.Cd, zeros(pd,n);...
          us.Cz, zeros(pz,n)];
    MS = [us.Bu, eye(n,n);...
          us.Ddu, zeros(pd,n);...
          us.Dzu, zeros(pz,n)];
    Mg = [zeros(n, pz);...
          zeros(pd, pz);...
          eye(pz, pz)];
    Mc = [us.Bw;...
          us.Ddw;...
          us.Dzw];
    Mt = [us.Bd, zeros(n,pd);...
          us.Ddd, eye(pd,pd);...
          us.Dzd, zeros(pz,pd)];
    lyap = MP*RP*MP' + MS*RS*MS' - tau*gamma2*(Mg*Mg') + tau*(Mc*Mc') - Mt*sepvar*Mt';
  end
else
  RP = [-P, zeros(n, 2*n); ...
        zeros(n, 2*n), P;...
        zeros(n, n), P, zeros(n, n)];
  if deltanul
    MP = [eye(n,n), us.A, zeros(n,n);...
          zeros(pz,n), us.Cz, zeros(pz,n);...
          zeros(n,n), -0.5*eye(n,n), eye(n,n)];
    MS = [us.Bu, zeros(n,n);...
          us.Dzu, zeros(pz,n);...
          zeros(n,mu), eye(n,n)];
    Mg = [zeros(n, pz);...
          eye(pz, pz);...
          zeros(n, pz)];
    Mc = [us.Bw;...
          us.Dzw;...
          zeros(n, mw)];
    lyap = MP*RP*MP' + MS*RS*MS' - tau*gamma2*(Mg*Mg') + tau*(Mc*Mc');
  else
    MP = [eye(n,n), us.A, zeros(n,n);...
          zeros(pd,n), us.Cd, zeros(pd,n);...
          zeros(pz,n), us.Cz, zeros(pz,n);...
          zeros(n,n), -0.5*eye(n,n), eye(n,n)];
    MS = [us.Bu, zeros(n,n);...
          us.Ddu, zeros(pd,n);...
          us.Dzu, zeros(pz,n);...
          zeros(n,mu), eye(n,n)];
    Mg = [zeros(n, pz);...
          zeros(pd, pz);...
          eye(pz, pz);...
          zeros(n, pz)];
    Mc = [us.Bw;...
          us.Ddw;...
          us.Dzw;...
          zeros(n, mw)];
    Mt = [us.Bd, zeros(n,pd);...
          us.Ddd, eye(pd,pd);...
          us.Dzw, zeros(pz,pd);...
          zeros(n,md+pd)];
    lyap = MP*RP*MP' + MS*RS*MS' - tau*gamma2*(Mg*Mg') + tau*(Mc*Mc') - Mt*sepvar*Mt';
  end
end

if isLMI;
  cstr = tag(lyap<= -varsglob{1,1},'Var Lyap <0');
else
  cstr{1} = lyap;
end
